package it.amattioli.guidate.browsing.calendar;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.text.SimpleDateFormat;
import java.util.Properties;

import org.apache.commons.lang.StringUtils;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zk.ui.util.GenericComposer;
import org.zkoss.zul.Column;
import org.zkoss.zul.Columns;
import org.zkoss.zul.Grid;
import org.zkoss.zul.Label;
import org.zkoss.zul.RowRenderer;

import it.amattioli.encapsulate.browsers.CalendarBrowser;
import it.amattioli.encapsulate.browsers.CalendarBrowser.IntervalType;
import it.amattioli.guidate.containers.BackBeans;

public class IntervalGridComposer extends GenericComposer {
	private Properties names = new Properties();
	private PropertyChangeListener intervalTypeChangeListener;
	
	public IntervalGridComposer() throws Exception {
		names.load(this.getClass().getResourceAsStream("names.properties"));
	}
	
	public void onCreate(Event evt) throws Exception {
		final Grid self = (Grid)evt.getTarget();
		final CalendarBrowser<?, ?> browser = (CalendarBrowser<?, ?>)BackBeans.findBackBean(self);
		self.appendChild(createColumnHeaders(browser));
		self.setRowRenderer(findRenderer(browser.getIntervalType()));
		self.setModel(new CalendarModel(browser));
		intervalTypeChangeListener = new PropertyChangeListener() {

			@Override
			public void propertyChange(PropertyChangeEvent evt) {
				if ("intervalType".equals(evt.getPropertyName())) {
					self.getColumns().setParent(null);
					self.appendChild(createColumnHeaders(browser));
					self.setRowRenderer(findRenderer(browser.getIntervalType()));
					self.setModel(new CalendarModel(browser));
				}
			}
			
		};
		browser.addPropertyChangeListener(intervalTypeChangeListener);
	}
	
	private RowRenderer findRenderer(IntervalType type) {
		/*
		if (IntervalType.DAY.equals(type)) {
			return new DayRenderer();
		}
		if (IntervalType.WEEK.equals(type)) {
			return new WeekRenderer();
		}
		if (IntervalType.MONTH.equals(type)) {
			return new MonthRenderer();
		}
		return null;
		*/
		try {
			return new CalendarGridRenderer();
		} catch(Exception e) {
			throw new RuntimeException(e);
		}
	}
	
	private Columns createColumnHeaders(CalendarBrowser<?, ?> browser) {
		Columns cols = new Columns();
		CalendarBrowser<?, ?> sbrowser = browser.subBrowser().subBrowser();
		addColumn(cols, sbrowser);
		while(sbrowser.hasNextInterval()) {
			sbrowser.nextInterval();
			addColumn(cols, sbrowser);
		}
		return cols;
	}
	
	private void addColumn(Columns cols, CalendarBrowser<?, ?> brw) {
		if (!StringUtils.isBlank(names.getProperty(brw.getInterval().getClass().getCanonicalName()+".column"))) {
			String format = names.getProperty(brw.getInterval().getClass().getCanonicalName()+".column");
			String colName = new SimpleDateFormat(format).format(brw.getInterval().getInitTime());
			Column col = new Column(colName);
			col.setParent(cols);
		}
	}
	
}